
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "gtksource-enumtypes.h"
/* enumerations from "gtksourcebuffer.h" */
#include "gtksourcebuffer.h"
GType
gtk_source_bracket_match_type_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_BRACKET_MATCH_NONE,
          "GTK_SOURCE_BRACKET_MATCH_NONE",
          "none" },
        { GTK_SOURCE_BRACKET_MATCH_OUT_OF_RANGE,
          "GTK_SOURCE_BRACKET_MATCH_OUT_OF_RANGE",
          "out-of-range" },
        { GTK_SOURCE_BRACKET_MATCH_NOT_FOUND,
          "GTK_SOURCE_BRACKET_MATCH_NOT_FOUND",
          "not-found" },
        { GTK_SOURCE_BRACKET_MATCH_FOUND,
          "GTK_SOURCE_BRACKET_MATCH_FOUND",
          "found" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceBracketMatchType"),
        values);
    }

  return the_type;
}
GType
gtk_source_change_case_type_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_CHANGE_CASE_LOWER,
          "GTK_SOURCE_CHANGE_CASE_LOWER",
          "lower" },
        { GTK_SOURCE_CHANGE_CASE_UPPER,
          "GTK_SOURCE_CHANGE_CASE_UPPER",
          "upper" },
        { GTK_SOURCE_CHANGE_CASE_TOGGLE,
          "GTK_SOURCE_CHANGE_CASE_TOGGLE",
          "toggle" },
        { GTK_SOURCE_CHANGE_CASE_TITLE,
          "GTK_SOURCE_CHANGE_CASE_TITLE",
          "title" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceChangeCaseType"),
        values);
    }

  return the_type;
}
GType
gtk_source_sort_flags_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GFlagsValue values[] = {
        { GTK_SOURCE_SORT_FLAGS_NONE,
          "GTK_SOURCE_SORT_FLAGS_NONE",
          "none" },
        { GTK_SOURCE_SORT_FLAGS_CASE_SENSITIVE,
          "GTK_SOURCE_SORT_FLAGS_CASE_SENSITIVE",
          "case-sensitive" },
        { GTK_SOURCE_SORT_FLAGS_REVERSE_ORDER,
          "GTK_SOURCE_SORT_FLAGS_REVERSE_ORDER",
          "reverse-order" },
        { GTK_SOURCE_SORT_FLAGS_REMOVE_DUPLICATES,
          "GTK_SOURCE_SORT_FLAGS_REMOVE_DUPLICATES",
          "remove-duplicates" },
        { 0, NULL, NULL }
      };

      the_type = g_flags_register_static (
        g_intern_static_string ("GtkSourceSortFlags"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcecompletion.h" */
#include "gtksourcecompletion.h"
GType
gtk_source_completion_error_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_COMPLETION_ERROR_ALREADY_BOUND,
          "GTK_SOURCE_COMPLETION_ERROR_ALREADY_BOUND",
          "already-bound" },
        { GTK_SOURCE_COMPLETION_ERROR_NOT_BOUND,
          "GTK_SOURCE_COMPLETION_ERROR_NOT_BOUND",
          "not-bound" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceCompletionError"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcecompletioncontext.h" */
#include "gtksourcecompletioncontext.h"
GType
gtk_source_completion_activation_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GFlagsValue values[] = {
        { GTK_SOURCE_COMPLETION_ACTIVATION_NONE,
          "GTK_SOURCE_COMPLETION_ACTIVATION_NONE",
          "none" },
        { GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE,
          "GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE",
          "interactive" },
        { GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED,
          "GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED",
          "user-requested" },
        { 0, NULL, NULL }
      };

      the_type = g_flags_register_static (
        g_intern_static_string ("GtkSourceCompletionActivation"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourceencoding.h" */
#include "gtksourceencoding.h"
GType
gtk_source_encoding_duplicates_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_ENCODING_DUPLICATES_KEEP_FIRST,
          "GTK_SOURCE_ENCODING_DUPLICATES_KEEP_FIRST",
          "first" },
        { GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST,
          "GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST",
          "last" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceEncodingDuplicates"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcefile.h" */
#include "gtksourcefile.h"
GType
gtk_source_newline_type_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_NEWLINE_TYPE_LF,
          "GTK_SOURCE_NEWLINE_TYPE_LF",
          "lf" },
        { GTK_SOURCE_NEWLINE_TYPE_CR,
          "GTK_SOURCE_NEWLINE_TYPE_CR",
          "cr" },
        { GTK_SOURCE_NEWLINE_TYPE_CR_LF,
          "GTK_SOURCE_NEWLINE_TYPE_CR_LF",
          "cr-lf" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceNewlineType"),
        values);
    }

  return the_type;
}
GType
gtk_source_compression_type_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_COMPRESSION_TYPE_NONE,
          "GTK_SOURCE_COMPRESSION_TYPE_NONE",
          "none" },
        { GTK_SOURCE_COMPRESSION_TYPE_GZIP,
          "GTK_SOURCE_COMPRESSION_TYPE_GZIP",
          "gzip" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceCompressionType"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcefileloader.h" */
#include "gtksourcefileloader.h"
GType
gtk_source_file_loader_error_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_FILE_LOADER_ERROR_TOO_BIG,
          "GTK_SOURCE_FILE_LOADER_ERROR_TOO_BIG",
          "too-big" },
        { GTK_SOURCE_FILE_LOADER_ERROR_ENCODING_AUTO_DETECTION_FAILED,
          "GTK_SOURCE_FILE_LOADER_ERROR_ENCODING_AUTO_DETECTION_FAILED",
          "encoding-auto-detection-failed" },
        { GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK,
          "GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK",
          "conversion-fallback" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceFileLoaderError"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcefilesaver.h" */
#include "gtksourcefilesaver.h"
GType
gtk_source_file_saver_error_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_FILE_SAVER_ERROR_INVALID_CHARS,
          "GTK_SOURCE_FILE_SAVER_ERROR_INVALID_CHARS",
          "invalid-chars" },
        { GTK_SOURCE_FILE_SAVER_ERROR_EXTERNALLY_MODIFIED,
          "GTK_SOURCE_FILE_SAVER_ERROR_EXTERNALLY_MODIFIED",
          "externally-modified" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceFileSaverError"),
        values);
    }

  return the_type;
}
GType
gtk_source_file_saver_flags_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GFlagsValue values[] = {
        { GTK_SOURCE_FILE_SAVER_FLAGS_NONE,
          "GTK_SOURCE_FILE_SAVER_FLAGS_NONE",
          "none" },
        { GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_INVALID_CHARS,
          "GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_INVALID_CHARS",
          "ignore-invalid-chars" },
        { GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME,
          "GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME",
          "ignore-modification-time" },
        { GTK_SOURCE_FILE_SAVER_FLAGS_CREATE_BACKUP,
          "GTK_SOURCE_FILE_SAVER_FLAGS_CREATE_BACKUP",
          "create-backup" },
        { 0, NULL, NULL }
      };

      the_type = g_flags_register_static (
        g_intern_static_string ("GtkSourceFileSaverFlags"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcegutterrenderer.h" */
#include "gtksourcegutterrenderer.h"
GType
gtk_source_gutter_renderer_state_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GFlagsValue values[] = {
        { GTK_SOURCE_GUTTER_RENDERER_STATE_NORMAL,
          "GTK_SOURCE_GUTTER_RENDERER_STATE_NORMAL",
          "normal" },
        { GTK_SOURCE_GUTTER_RENDERER_STATE_CURSOR,
          "GTK_SOURCE_GUTTER_RENDERER_STATE_CURSOR",
          "cursor" },
        { GTK_SOURCE_GUTTER_RENDERER_STATE_PRELIT,
          "GTK_SOURCE_GUTTER_RENDERER_STATE_PRELIT",
          "prelit" },
        { GTK_SOURCE_GUTTER_RENDERER_STATE_SELECTED,
          "GTK_SOURCE_GUTTER_RENDERER_STATE_SELECTED",
          "selected" },
        { 0, NULL, NULL }
      };

      the_type = g_flags_register_static (
        g_intern_static_string ("GtkSourceGutterRendererState"),
        values);
    }

  return the_type;
}
GType
gtk_source_gutter_renderer_alignment_mode_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL,
          "GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL",
          "cell" },
        { GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST,
          "GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST",
          "first" },
        { GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_LAST,
          "GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_LAST",
          "last" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceGutterRendererAlignmentMode"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcespacedrawer.h" */
#include "gtksourcespacedrawer.h"
GType
gtk_source_space_type_flags_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GFlagsValue values[] = {
        { GTK_SOURCE_SPACE_TYPE_NONE,
          "GTK_SOURCE_SPACE_TYPE_NONE",
          "none" },
        { GTK_SOURCE_SPACE_TYPE_SPACE,
          "GTK_SOURCE_SPACE_TYPE_SPACE",
          "space" },
        { GTK_SOURCE_SPACE_TYPE_TAB,
          "GTK_SOURCE_SPACE_TYPE_TAB",
          "tab" },
        { GTK_SOURCE_SPACE_TYPE_NEWLINE,
          "GTK_SOURCE_SPACE_TYPE_NEWLINE",
          "newline" },
        { GTK_SOURCE_SPACE_TYPE_NBSP,
          "GTK_SOURCE_SPACE_TYPE_NBSP",
          "nbsp" },
        { GTK_SOURCE_SPACE_TYPE_ALL,
          "GTK_SOURCE_SPACE_TYPE_ALL",
          "all" },
        { 0, NULL, NULL }
      };

      the_type = g_flags_register_static (
        g_intern_static_string ("GtkSourceSpaceTypeFlags"),
        values);
    }

  return the_type;
}
GType
gtk_source_space_location_flags_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GFlagsValue values[] = {
        { GTK_SOURCE_SPACE_LOCATION_NONE,
          "GTK_SOURCE_SPACE_LOCATION_NONE",
          "none" },
        { GTK_SOURCE_SPACE_LOCATION_LEADING,
          "GTK_SOURCE_SPACE_LOCATION_LEADING",
          "leading" },
        { GTK_SOURCE_SPACE_LOCATION_INSIDE_TEXT,
          "GTK_SOURCE_SPACE_LOCATION_INSIDE_TEXT",
          "inside-text" },
        { GTK_SOURCE_SPACE_LOCATION_TRAILING,
          "GTK_SOURCE_SPACE_LOCATION_TRAILING",
          "trailing" },
        { GTK_SOURCE_SPACE_LOCATION_ALL,
          "GTK_SOURCE_SPACE_LOCATION_ALL",
          "all" },
        { 0, NULL, NULL }
      };

      the_type = g_flags_register_static (
        g_intern_static_string ("GtkSourceSpaceLocationFlags"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourcestylescheme.h" */
#include "gtksourcestylescheme.h"
GType
gtk_source_style_scheme_kind_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_STYLE_SCHEME_KIND_LIGHT,
          "GTK_SOURCE_STYLE_SCHEME_KIND_LIGHT",
          "light" },
        { GTK_SOURCE_STYLE_SCHEME_KIND_DARK,
          "GTK_SOURCE_STYLE_SCHEME_KIND_DARK",
          "dark" },
        { GTK_SOURCE_STYLE_SCHEME_KIND_LIGHT_ONLY,
          "GTK_SOURCE_STYLE_SCHEME_KIND_LIGHT_ONLY",
          "light-only" },
        { GTK_SOURCE_STYLE_SCHEME_KIND_DARK_ONLY,
          "GTK_SOURCE_STYLE_SCHEME_KIND_DARK_ONLY",
          "dark-only" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceStyleSchemeKind"),
        values);
    }

  return the_type;
}
/* enumerations from "gtksourceview.h" */
#include "gtksourceview.h"
GType
gtk_source_view_gutter_position_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_VIEW_GUTTER_POSITION_LINES,
          "GTK_SOURCE_VIEW_GUTTER_POSITION_LINES",
          "lines" },
        { GTK_SOURCE_VIEW_GUTTER_POSITION_MARKS,
          "GTK_SOURCE_VIEW_GUTTER_POSITION_MARKS",
          "marks" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceViewGutterPosition"),
        values);
    }

  return the_type;
}
GType
gtk_source_smart_home_end_type_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GTK_SOURCE_SMART_HOME_END_DISABLED,
          "GTK_SOURCE_SMART_HOME_END_DISABLED",
          "disabled" },
        { GTK_SOURCE_SMART_HOME_END_BEFORE,
          "GTK_SOURCE_SMART_HOME_END_BEFORE",
          "before" },
        { GTK_SOURCE_SMART_HOME_END_AFTER,
          "GTK_SOURCE_SMART_HOME_END_AFTER",
          "after" },
        { GTK_SOURCE_SMART_HOME_END_ALWAYS,
          "GTK_SOURCE_SMART_HOME_END_ALWAYS",
          "always" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GtkSourceSmartHomeEndType"),
        values);
    }

  return the_type;
}

/* Generated data ends here */

